from time import sleep
import pytest
from site24x7.connector.redis_client import RedisClient
from site24x7.connector.database_client import DatabaseClient
from site24x7.connector.cache_client import CacheClient
from site24x7.connector.request_client import RequestClient
from site24x7.utils.util import *


def testRedisClient():
    class RedisInst(RedisClient):
        def clientSize(self):
            return 2
    RedisInst = RedisInst(host='localhost', port=6379, db=0, charset="utf-8", decode_responses=True)
    RedisInst.redis.set('test', 'test', ex=6, nx=True, xx=False)
    RedisInst.redis.execute_command('SET test_column test_value EX 5 NX')
    assert RedisInst.redis.get('test_column') == 'test_value'
    assert RedisInst.redis.get('test') == 'test'
    assert RedisInst.clientSize() == 2
    sleep(6)
    try:
        RedisInst.redis.get('test_column')
    except KeyError:
        assert 1 == 1
    try:
        RedisInst.redis.get('test')
    except KeyError:
        assert 1 == 1
    print("Redis client... Passed")


# def testDatabaseClient():
#     class DatabaseInst(DatabaseClient):
#         def databaseType(self):
#             return "msSQL"
#     DatabaseInst = DatabaseInst(server='localhost', database='test', username='test', password='password', driverLocation='C:\\Program Files\\Microsoft SQL Server\\110\\Tools\\ODBC\\ODBC.INI')
#     assert DatabaseInst.databaseType() == "msSQL"
#     print("client... Passed")


# def testCacheClient():
#     class CacheInst(CacheClient):
#         pass
#     # create dummy json file
#     expected_output = {'test': 'test'}
#     instance_1 = CacheInst(cacheFileFullPath="/home/kali/Desktop/dummy.json", force=True)
#     instance_1.save(expected_output)
#     instance_2 = CacheInst(cacheFileFullPath="/home/kali/Desktop/dummy.json", force=False)
#     assert instance_2.dict==expected_output
#     print("client... Passed")

# def testRequestClient():

#     client = RequestClient()
#     req = client.get("https://www.google.com", headers={"kaka":"true"})
#     print(req);
#     print(req.status_code);
#     print(req.text);
#     print(req.headers);

if __name__ == "__main__":
    testRedisClient()
    # testDatabaseClient()
    # testCacheClient()



import redis
from .models.model import ClientModel

class RedisClient(ClientModel):
    ''' redis client is a client model that is used to get and set data from a redis server '''
    def __init__(self, host, port, db, *args, charset="utf-8", decode_responses=True, **kwargs):
        self.host=host
        self.port=port
        self.db=db
        self.charset=charset
        self.decode_responses=decode_responses
        self.redis=redis.Redis(
            host=self.host,
            port=self.port,
            db=self.db,
            charset=self.charset,
            decode_responses=self.decode_responses
        )
        super().__init__(*args, **kwargs)

    def hset(
        self,
        array_name:str,
        key:str,
        value:str
    ):
        ''' Set a key/value pair in a hash '''
        self.redis.hset(array_name, key, value)

    def hgetall(self, array_name:str) -> list:
        ''' Get all the values in a hash '''
        return self.redis.hgetall(array_name)

    def set_dict_using_global_selector(self, global_selector:str, array_name:str, set_item_key:str, set_as:str="string"):
        ''' Set a key/value pair into the main dictionary '''
        array = self.hgetall(array_name)
        for key in array:
            if key.startswith(global_selector):
                if set_as == "string":
                    self.__setitem__(set_item_key, array[key])
                elif set_as == "int":
                    self.__setitem__(set_item_key, int(array[key]))
                elif set_as == "float":
                    self.__setitem__(set_item_key, float(array[key]))
                elif set_as == "bool":
                    self.__setitem__(set_item_key, bool(array[key]))
                else:
                    raise ValueError("Invalid set_as value")
                return
        raise KeyError("Key not found")



import os
import json
import argparse # use argparse to parse the arguments
from subprocess import call
from time import strftime
from dotenv import load_dotenv
from site24x7_plugin_helper.connector.database_client import DatabaseClient
from site24x7_plugin_helper.connector.redis_client import RedisClient
from site24x7_plugin_helper.connector.request_client import RequestClient
from site24x7_plugin_helper.utils.util import validatePluginData

# setting up the argument parser
parser = argparse.ArgumentParser(description='This is a helper script for Site24x7 plugins.')
parser.add_argument('--plugin',
    help='Unique plugin name that will be shown in site24x7 plugin list',
    required=True,
    type=str)
parser.add_argument('--query',
    help='MSSQL query to be executed',
    required=False,
    type=str)
parser.add_argument('--version',
    help="Which step of the plugin is this? (1, 2, 3, etc.)",
    required=False,
    type=int)
parser.add_argument('--server', help='Most of the time an IPv4 or IPv6', type=str)
##
parser.add_argument('--connection_type', help='Connection types: redis, request, database', required=True, type=str)
parser.add_argument('--database', help='Database name that belongs to the server', default=None, type=str)
parser.add_argument('--redis', help='Redis connection', default=None, type=str)
parser.add_argument('--request', help='Http(s) connection', default=None, type=str)
##
parser.add_argument('--env', help='Path to .env file', default=None, type=str)
parser.add_argument('--username', help='Username for connection_type', default=None, type=str)
parser.add_argument('--password', help='Password for connection_type', default=None, type=str)
parser.add_argument('--database_driver_path', help="database driver's location if exists", default=None, type=str)
# default="/usr/local/lib/python3.6/dist-packages/pyodbc/drivers/pyodbc.so", type=str)

# load the environment variables from file location if exists
if parser.parse_args().env:
    load_dotenv(dotenv_path=parser.parse_args().env)

def special_strftime():
    ''' expected output: Aug 22nd, 2022 '''
    dic = {
        '01':'st,','21':'st,','31':'st,',
        '02':'nd,','22':'nd,',
        '03':'rd,','23':'rd,'
    }
    x = strftime('%b %d')
    year = strftime(' %Y')
    return x + dic.get(x[-2:], 'th,') + year

if __name__ == "__main__":
    this_plugin_name = parser.parse_args().plugin or os.environ.get("THIS_PLUGIN_NAME")
    this_plugin_name = this_plugin_name.replace("/", "")
    query = parser.parse_args().query or os.environ.get("QUERY")
    version = parser.parse_args().version or os.environ.get("VERSION")
    server = parser.parse_args().server or os.environ.get("SERVER")
    username = parser.parse_args().username or os.environ.get("USERNAME")
    password = parser.parse_args().password or os.environ.get("PASSWORD")
    connection_type = parser.parse_args().connection_type or os.environ.get("CONNECTION_TYPE")
    database = None
    redis = None
    http = None
    if connection_type == "database":
        database = parser.parse_args().database or os.environ.get("DATABASE")
        database_driver_path = parser.parse_args().database_driver_path or os.environ.get("DATABASE_DRIVER_PATH")
    elif connection_type == "redis":
        redis = parser.parse_args().redis or os.environ.get("REDIS")
    elif connection_type == "request":
        request = parser.parse_args().request or os.environ.get("REQUEST")
    if version is None:
        version = "1"
    if server is None:
        raise Exception("Server is not defined")
    if database is None:
        raise Exception("Database is not defined")
    if username is None:
        raise Exception("Username is not defined")
    if password is None:
        raise Exception("Password is not defined")
    cache = {
        "database":database,
        "query":query,
        "this_plugin_name":this_plugin_name,
        "version":version,
    }
    if database:
        # create a DatabaseClient instance
        client = DatabaseClient(server, database, username, password, database_driver_path)
        cursor = client.cursor
        print("connnected")
        # existing query that returns all rows from the table for caching
        rows = client.fetchall(query)
        for i in enumerate(rows):
            key = rows[i][1]
            value = rows[i][2]
            cache[key] = value
    elif redis:
        client = RedisClient(server, database, username, password)
        print("connnected")
    elif request:
        client = RequestClient(server, database, username, password)
        print("connnected")
    ## validate data before saving to cache
    if not validatePluginData(cache):
        raise Exception("Data validation failed")
    # save cache to file as this_plugin_name
    cache_file_name = "/tmp/" + this_plugin_name + ".cache"
    open(cache_file_name, 'w', encoding='utf8').write(
        json.dumps(cache, indent=4)
    )
    print(json.dumps(cache, indent=4, sort_keys=True))

    SITE24X7_LOCATION = "/opt/site24x7/monagent/plugins"
    folder_destination = os.path.join(SITE24X7_LOCATION, this_plugin_name)
    file_destination = os.path.join(folder_destination, this_plugin_name + ".py")
    print("folder_destination: " + folder_destination)
    print("file_destination: " + file_destination)
    # create a folder if not exists
    if not os.path.exists(folder_destination):
        call(f"mkdir {folder_destination}", shell=True)

    # create a python file if not exists
    if not os.path.isfile(file_destination):
        call(f"touch {file_destination}", shell=True)
        open(file_destination, 'w', encoding="utf8").write(f"""#!/usr/bin/env python3
'''
   this file is generated using Jenkins, and it is a cron-task
'''
from site24x7_plugin_helper.connector.cache_client import CacheClient

CLIENT = CacheClient(
    "{cache_file_name}",
    date_created="{special_strftime()}",
    ignored_list=["query", "this_plugin_name"]
)

def metric_collector():
    data = {{}}
    try:
        CLIENT.set_metric_types()
        data.update(CLIENT.dict)
    except Exception as err:
        data["error"] = str(err)
    return data

if __name__ == "__main__":
    result = metric_collector()
    print(CLIENT.dumps(result))
""")
    else:
        # replace the file with new version
        pass



import json
from datetime import datetime
import requests
AVAILABLE_SCOPES = "Site24x7.Account.Read, Site24x7.Account.Create, Site24x7.Account.Update, Site24x7.Account.Delete, Site24x7.Account.All, Site24x7.Reports.Read, Site24x7.Reports.Create, Site24x7.Reports.Update, Site24x7.Reports.Delete, Site24x7.Reports.All,  Site24x7.Operations.Read, Site24x7.Operations.Create, Site24x7.Operations.Update, Site24x7.Operations.Delete, Site24x7.Operations.All, Site24x7.Msp.Read, Site24x7.Msp.Create, Site24x7.Msp.Update, Site24x7.Msp.Delete, Site24x7.Msp.All, Site24x7.Bu.Read, Site24x7.Bu.Create, Site24x7.Bu.Update, Site24x7.Bu.Delete, Site24x7.Bu.All"
READ_SOPES = "Site24x7.Account.Read, Site24x7.Reports.Read, Site24x7.Operations.Read, Site24x7.Admin.Read"
ALL_SOPES = "Site24x7.Account.All, Site24x7.Reports.All, Site24x7.Operations.All, Site24x7.Admin.All"
current_status_type = [
    'monitor_id',
    'name',
    'monitor_type',
    'status',
]
class ZohoConnection:
    ''' Zoho connection class for Oath2.0 authentication workflow
        :param client_id: Zoho client id
        :param client_secret: Zoho client
        :param code: Zoho authorization code
        :param refresh_token: Zoho refresh token
        :param access_token: Zoho access token
        :param cache: Use cache file, default True, if False will ignore cache file.
            cache will be saved into .cache after each refresh or authorization code function
    '''
    MONITOR_GRUPS = {}
    MONITORS = {}
    MSP_CUSTOMERS = {}
    DEVICE_KEY = None
    def __init__(self, client_id=None, client_secret=None, code=None, refresh_token=None, access_token=None, cache=True):
        if cache:
            try:
                self.pull_data_from_cache()
            except FileNotFoundError:
                cache = False
        if not cache:
            self.client_id = client_id
            self.client_secret = client_secret
            self.refresh_token = refresh_token
            self.headers = {
                "Content-Type": "application/x-www-form-urlencoded;charset=UTF-8",
                "Accept": "application/json; version=2.0"
            }
            self.access_token = access_token
            self.token_type = None
            # 1 minute
            self.expration_time = datetime.now().timestamp() + 60
            self.expration_time_redable = datetime.fromtimestamp(self.expration_time).strftime('%Y-%m-%d %H:%M:%S')
            self.api_domain = "https://www.site24x7.com"
            self.auth_domain = 'https://accounts.zoho.com'
            self.code = code
            if not self.refresh_token and not self.code:
                raise Exception('No refresh token or code provided')
            if not self.refresh_token and self.code:
                self.authorization_code(self.code)
            elif not access_token:
                self.refresh()
            else:
                self.headers['Authorization'] = 'Zoho-oauthtoken ' + self.access_token
            try:
                # this is still experimental
                self.set_device_key()
            except Exception as exp:
                print(exp)

    def pull_data_from_cache(self):
        ''' Pull data from cache file '''
        with open('.cache', 'r', encoding="utf-8") as cache_file:
            data = json.loads(cache_file.read())
            self.__dict__ = data

    def authorization_code(self, code):
        ''' Get access token from authorization code '''
        url = self.auth_domain + '/oauth/v2/token'
        data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'code': code,
            'grant_type': 'authorization_code'
        }
        response = requests.post(url, data=data, timeout=10, headers=self.headers)
        if response.status_code == 200:
            response = json.loads(response.text)
            if "error" in response:
                raise Exception(response["error"])
            self.access_token = response['access_token']
            self.token_type = response['token_type']
            self.refresh_token = response['refresh_token']
            self.expration_time = datetime.now().timestamp() + response['expires_in']
            self.expration_time_redable = datetime.fromtimestamp(self.expration_time).strftime('%Y-%m-%d %H:%M:%S')
            self.headers['Authorization'] = 'Zoho-oauthtoken ' + self.access_token

    def refresh(self):
        ''' Refresh the access token '''
        self.did_expire()
        url = self.auth_domain + '/oauth/v2/token'
        data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'refresh_token': self.refresh_token,
            'grant_type': 'refresh_token'
        }
        response = requests.post(url, data=data, timeout=10, headers=self.headers)
        if response.status_code == 200:
            response = json.loads(response.text)
            if "error" in response:
                raise Exception(response["error"])
            self.access_token = response['access_token']
            self.token_type = response['token_type']
            self.expration_time = datetime.now().timestamp() + response['expires_in']
            self.expration_time_redable = datetime.fromtimestamp(self.expration_time).strftime('%Y-%m-%d %H:%M:%S')
            self.headers['Authorization'] = 'Zoho-oauthtoken ' + self.access_token
        else:
            raise Exception('Error refreshing token')
        # save all data to a text file
        self.save_data_into_file()

    def save_data_into_file(self):
        ''' Save all data into a file '''
        with open('.cache', 'w', encoding="utf-8") as cache_file:
            self.timestamp = datetime.now().timestamp()
            self.timestamp_radable = datetime.fromtimestamp(self.timestamp).strftime('%Y-%m-%d %H:%M:%S')
            cache_file.write(json.dumps(self.__dict__, indent=4))

    def get(self, url, params:dict=None, timeout:int=20) -> dict:
        ''' GET request '''
        self.refresh()
        url = self.api_domain + url
        headers = self.headers
        response = requests.get(url, params=params, headers=headers, timeout=20)
        json_response = json.loads(response.text)
        if response.status_code == 200:
            return json_response
        else:
            code = json_response['code']
            message = json_response['message']
            raise requests.exceptions.HTTPError(f'Error getting response with status {response.status_code}, code: {code}, message: {message}')

    def put(self, url, params:dict=None) -> dict:
        ''' PUT request '''
        self.refresh()
        url = self.api_domain + url
        headers = self.headers
        response = requests.put(url, params=params, headers=headers, timeout=20)
        json_response = json.loads(response.text)
        if response.status_code == 200:
            return json_response
        else:
            code = json_response['code']
            message = json_response['message']
            raise requests.exceptions.HTTPError(f'Error getting response with status {response.status_code}, code: {code}, message: {message}')

    def get_current_status_all(self):
        ''' Get all current from site24x7 status '''
        url = '/api/current_status'
        response = self.get(url)
        for i in response['data']['monitor_groups']:
            group_id = i['group_id']
            group_name = i['group_name']
            status = i['status']
            if "monitors" in i:
                monitors = i['monitors']
            else:
                monitors = []
            if "downMonitorsCount" in i:
                downMonitorsCount = i['downMonitorsCount']
            else:
                downMonitorsCount = 0
            if "criticalMonitorsCount" in i:
                criticalMonitorsCount = i['criticalMonitorsCount']
            else:
                criticalMonitorsCount = 0
            if "threshold_count" in i:
                threshold_count = i['threshold_count']
            else:
                threshold_count = 0
            if "troubleMonitorsCount" in i:
                troubleMonitorsCount = i['troubleMonitorsCount']
            else:
                troubleMonitorsCount = 0
            self.MONITOR_GRUPS[group_id] = {
                'group_name': group_name,
                'monitors': monitors,
                'status': status,
                'downMonitorsCount': downMonitorsCount,
                'criticalMonitorsCount': criticalMonitorsCount,
                'threshold_count': threshold_count,
                'troubleMonitorsCount': troubleMonitorsCount
            }
        for i in response['data']['monitors']:
            monitor_id = i['monitor_id']
            name = i['name']
            attributeName = i['attributeName']
            monitor_type = i['monitor_type']
            last_polled_time = i['last_polled_time']
            status = i['status']
            self.MONITORS[monitor_id] = {
                'name': name,
                'monitor_type': monitor_type,
                'status': status
            }

    def get_all_monitors(self):
        ''' Get all monitors from site24x7 '''
        url = '/api/monitors'
        response = self.get(url, timeout=60)
        code = response['code']
        message = response['message']
        if message != 'success':
            raise requests.exceptions.HTTPError(f'Error on all monitors with status {response.status_code}, code: {code}, message: {message}')
        for i in response['data']:
            monitor_id = i['monitor_id']
            monitor_type = i['type']
            if 'name' in i:
                name = i['name']
            else:
                name = i['display_name']
            if 'status' in i:
                status = i['status']
            else:
                status = i['state']
            self.MONITORS[monitor_id] = {
                'name': name,
                'monitor_type': monitor_type,
                'status': status,
            }
        return self.MONITORS

    def get_up_monitors(self):
        ''' Get all up monitors from existing monitor groups '''
        up_monitors = []
        for key, values in self.MONITORS.items():
            if values['status'] == 1:
                _dict = self.MONITORS[key]
                _dict.update({"id": key})
                up_monitors.append(_dict)
        return up_monitors

    def get_monitor_by_name(self, name):
        ''' Get monitor by name '''
        for key, values in self.MONITORS.items():
            if values['name'] == name:
                _dict = self.MONITORS[key]
                _dict.update({"id": key})
                return _dict
        return None

    def get_monitors_name_includes(self, name):
        ''' Get monitors by name '''
        monitors = []
        for key, values in self.MONITORS.items():
            if name.lower() in values['name'].lower():
                _dict = self.MONITORS[key]
                _dict.update({"id": key})
                monitors.append(_dict)
        return monitors

    def get_msp_customers(self):
        ''' Get all msp customers
            *This function is still in development. '''
        url = '/api/msp/customers'
        response = self.get(url)
        for i in response['data']:
            name = i['name']
            zaaid = i['zaaid']
            self.MSP_CUSTOMERS[name] = zaaid

    def did_refresh(self):
        ''' Check if the token has been refreshed '''
        return self.access_token is not None

    def did_expire(self):
        ''' Raise error if the token has expired '''
        time = datetime.now().timestamp()
        experition_time = (self.expration_time - time) / 60
        if experition_time < 5:
            print(f'Access expires in: {experition_time} minutes')
        if time > self.expration_time:
            raise Exception('Token expired')

    def set_device_key(self):
        ''' Set the device key '''
        self.DEVICE_KEY = self.get_device_key()

    def get_device_key(self):
        ''' Get device key from site24x7
            *This function is still in development. '''
        url = '/api/device_key'
        response = self.get(url)
        print("get_device_key", response)
        return response['data']['device_key']

    def poll_monitor(self, monitor_id):
        ''' Poll a monitor. Initializes polling and returns the status of polling '''
        url = f'/api/monitor/poll_now/{monitor_id}'
        response = self.get(url)
        return response['data']['status']

    def poll_status(self, monitor_id):
        ''' Get the status of polling '''
        url = f'/api/monitor/status_poll_now/{monitor_id}'
        response = self.get(url)
        return response['data']['status']

    def activate_monitor(self, monitor_id):
        ''' Activate a monitor '''
        url = f'/api/monitors/activate/{monitor_id}'
        response = self.put(url)
        return response['message']
    
    def suspend_monitor(self, monitor_id):
        ''' Suspend a monitor '''
        url = f'/api/monitors/suspend/{monitor_id}'
        response = self.put(url)
        return response['message']
    
    def get_monitor_count(self):
        ''' Get the number of monitors from site24x7 '''
        url = f"/api/monitors/status/count"
        response = self.get(url)
        if response['message'] == "success":
            data = response['data']
            down_count = data['down']['count']
            up_count = data['up']['count']
            critical_count = data['critical']['count']
            trouble_count = data['trouble']['count']
            suspended_count = data['suspended']['count']
            maintenance_count = data['maintenance']['count']
            discovery_count = data['discovery']['count']
            total_count = data['total']['count']
            print("####################")
            print(f"Total: {total_count}")
            print(f"Up: {up_count}")
            print(f"Down: {down_count}")
            print(f"Critical: {critical_count}")
            print(f"Trouble: {trouble_count}")
            print(f"Suspended: {suspended_count}")
            print("####################")
            return data
        else:
            return None

    def suspend_given_plugin_name_list(self, name_list:list):
        ''' Suspend a list of monitors '''
        for i in name_list:
            monitor = self.get_monitor_by_name(i)
            if not monitor:
                print(f"Monitor {i} not found")
                continue
            self.suspend_monitor(monitor['id'])
            print(f"Suspended {i['name']}")

    def activate_given_plugin_name_list(self, name_list:list):
        ''' Activate a list of monitors '''
        for i in name_list:
            monitor = self.get_monitor_by_name(i)
            if not monitor:
                print(f"Monitor {i} not found")
                continue
            self.activate_monitor(monitor['id'])
            print(f"Suspended {i['name']}")
